#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
import time

from confluent_kafka import Producer
from mobio.libs.Singleton import Singleton

from .config import MobioEnvironment, ConsumerTopic
from .constant import ParamMessage


@Singleton
class MobioNotifySDK(object):
    PREFIX_LOGGER = "[NOTIFY_SDK]"
    PREFIX_LOGGER_ERR = "[NOTIFY_SDK_ERR]"

    def __init__(self):
        self.source = ""
        self.email_merchant_id = ""
        self.email_sender_id = ""
        self.email_sender_domain = ""
        self.email_sender_type = ""
        self.email_sender_name = ""

    def config(self, source, email_merchant_id, email_sender_id, email_sender_domain,
               email_sender_type, email_sender_name):
        self.source = source
        self.email_merchant_id = email_merchant_id
        self.email_sender_id = email_sender_id
        self.email_sender_domain = email_sender_domain
        self.email_sender_type = email_sender_type
        self.email_sender_name = email_sender_name

    def validate_data_send_notify(self, **kwargs):
        """
        Kiểm tra các trường thông tin message thông báo và dữ liệu khởi tạo
        :param kwargs:
        :return:
        """
        if not self.source:
            raise Exception("{} {} is require".format(MobioNotifySDK.PREFIX_LOGGER_ERR, "source"))

        if not self.email_sender_type:
            raise Exception("{} {} is require".format(MobioNotifySDK.PREFIX_LOGGER_ERR, "email_sender_type"))

        if not self.email_merchant_id:
            raise Exception("{} {} is require".format(MobioNotifySDK.PREFIX_LOGGER_ERR, "email_merchant_id"))

        if not self.email_sender_id:
            raise Exception("{} {} is require".format(MobioNotifySDK.PREFIX_LOGGER_ERR, "email_sender_id"))

        if not self.email_sender_domain:
            raise Exception("{} {} is require".format(MobioNotifySDK.PREFIX_LOGGER_ERR, "email_sender_domain"))

        print("{}: validate_data_send_notify: kwargs: {}".format(MobioNotifySDK.PREFIX_LOGGER, kwargs))
        list_field_validate = ParamMessage.LIST_FIELD_REQUIRED
        for field in list_field_validate:
            if not kwargs.get(field):
                raise Exception("{} {} is require".format(MobioNotifySDK.PREFIX_LOGGER_ERR, field))

        if kwargs.get(ParamMessage.ACCOUNT_IDS) and isinstance(kwargs.get(ParamMessage.ACCOUNT_IDS), list) is False:
            raise Exception("{} account_ids is list".format(MobioNotifySDK.PREFIX_LOGGER_ERR))

    def send_message_notify_sdk(self, merchant_id, key_config, account_ids, **kwargs):
        """
        Thực hiện việc gửi message thông báo đến Topic Notify SDK

        :param merchant_id: Id thương hiệu
        :param key_config: kiểu thông báo (được cấu hình trên admin) (ex: 'jb_create_by_me_end',
        'jb_estimate_target_size')
        :param account_ids: danh sách id nhân viên (id nhân viên của module ADMIN)
        :param kwargs: Thông tin các field nội dung thông báo toàn kênh định dạng key=value
        Truyền định dạng send_message_notify_sdk(merchant_id, key_config, account_ids, deal_count=5) lên hàm xử lý.
        SDK sẽ tự bắt thông tin field để thực hiện replace theo nội dung đã được cấu hình trên Admin.
        EX: title: Thông báo đơn hàng tồn quá **deal_count** đơn.
            content: Thông báo đơn hàng tồn quá **deal_count** đơn.
                    Vui lòng phân công cho nhân viên xử lý.
        - Field chuẩn hoá gửi kênh email truyền thêm các field sau(vì là kwargs vui lòng truyền đúng định dạng key=value):
            + email_file_alert: boolean (File thông báo ko có button giá trị = False, có button giá trị = True,
             mặc định giá trị = False)
            + email_button_name: string (Tên button email, mặc định ko truyền lên filed này sẽ có giá trị = "Tải file",
                                        email_file_alert = False bỏ qua field này)
            + email_url_file: string (đường dẫn khi submit vào button email; email_file_alert = False bỏ qua field này)
            + email_subject: string (nếu muốn tự custom tiêu đề email)
            + email_content: string (nếu muốn tự custom nội dung email)
        - Field chuẩn hoá gửi kênh socket truyền thêm các field sau (vì là kwargs vui lòng truyền đúng định dạng key=value):
            + socket_title: string (Nếu muốn tự custom tiêu đề gửi đến các kênh App, Browser, Web)
            + socket_content: string (Nếu muốn tự custom nội dung gửi đến các kênh App, Browser, Web)
        """
        start_time = time.time()

        MobioNotifySDK().validate_data_send_notify(merchant_id=merchant_id, key_config=key_config,
                                                   account_ids=account_ids)

        data_send_producer = {
            ParamMessage.SOURCE: self.source,
            ParamMessage.EMAIL_SENDER_TYPE: self.email_sender_type,
            ParamMessage.EMAIL_MERCHANT_ID: self.email_merchant_id,
            ParamMessage.EMAIL_SENDER_ID: self.email_sender_id,
            ParamMessage.EMAIL_SENDER_DOMAIN: self.email_sender_domain,
            ParamMessage.EMAIL_SENDER_NAME: self.email_sender_name,
            ParamMessage.MERCHANT_ID: merchant_id,
            ParamMessage.KEY_CONFIG: key_config,
            ParamMessage.ACCOUNT_IDS: account_ids
        }
        data_kwargs = dict()
        if kwargs:
            for key, value in kwargs.items():
                data_kwargs[key] = value

        data_send_producer.update({
            ParamMessage.DATA_KWARGS: data_kwargs
        })

        MobioNotifySDK.push_message_to_topic_receive_sdk(json.dumps(data_send_producer))
        print("{}: send_message_notify_sdk: time_process:: {}".format(
            MobioNotifySDK.PREFIX_LOGGER, time.time() - start_time))

    @staticmethod
    def push_message_to_topic_receive_sdk(data):
        """
        Bắn message đến topic SDK nhận tin xử lý
        :param data:
        :return:
        """
        conf = {
            "request.timeout.ms": 20000,
            "bootstrap.servers": MobioEnvironment.KAFKA_BROKER,
        }

        producer = Producer(conf)

        topic_sdk = ConsumerTopic.TOPIC_NOTIFY_SDK_RECEIVE_MESSAGE
        producer.produce(topic_sdk, data)
        producer.poll(0)
        print("{}: push_message_to_topic: topic: {}".format(MobioNotifySDK.PREFIX_LOGGER, topic_sdk))
        print("{}: push_message_to_topic: message: {}".format(MobioNotifySDK.PREFIX_LOGGER, data))
        producer.flush()
